% SDPAD: A collection C-Mex Language (MATLAB) codes for solving the following
% 	 semidefinite programming (SDP) problems:
%	-- Theta problem
%	-- Theta_plus problem
%	-- Frequencey assignment problem (FAP)
%	-- Binary integer programming problem (BIQ)
%
% -------------------------------------------------------------------------
% 1. Introduction
%
% Thank you for downloading SDPAD! SDPAD can be modified to solve
%     	min <C,X>, A(X) = b, X positive semidefinite
% or 	min <C,X>, A(X) = b, X positive semidefinite, X nonnegative 
% or 	min <C,X>, A(X) = b, B(X) >= d, X positive semidefinite, X nonnegative 
% by using an alternating direction augmented Lagrangian algorithm without
% transforming the original SDPs to the standard form SDPs.
%
% A detailed explanation of our algorithm is in:
% @TECHREPORT{AltSDP-WenGoldfarbYin-2009, 
%	author = {Wen, Zaiwen and Goldfarb, Donald and Yin, Wotao}, 
%	title = {Alternating Direction Augmented Lagrangian Methods for semidefinite programming}, 
%	institution = {Dept of IEOR, Columbia University}, 
%	year = {2009} }
% 
% -------------------------------------------------------------------------
% 2. Installation and Setup
%  -- Download, unzip and install C-Mex code "SDPAD.zip": 
%   Run "Installmex.m" in MATLAB to install the c-mex files.
%   This script is tested on versions 2006b and 2009a in Linux.	
%
%  -- Download and unzip the collection of test problems "AltSDPData.zip"
% -------------------------------------------------------------------------
%
% 3. Usage:
%
% *Driver routines: 
% 	Theta problem:		TestThetaG_HammingProbTable_theta.m
% 	Theta_plus problem:	TestThetaG_HammingProbTable_Plus.m
%	FAP:			TestFAPTable.m
%	BIQ:			TestBIQTable.m
%
%
% *Computational routines:
% 	Theta problem:		mexAltSDP_Theta.c	
% 	Theta_plus problem: 	mexAltSDP_ThetaPlus.c	
%	FAP:			mexAltSDP_FAP.c
%	BIQ:			mexAltSDP_BIQ.c	
%
% ** Since SDPAD are customized to solve the above four problems, the format of
% the input data is designed to take advantage the special structures of these
% problems. Please refer to the computational routines for the detailed
% information.
%
% *Examples for generating input data:
% 	Theta(_plus) problem:	GenTheta_mat.m, GenTheta_dat_s.m, GenIndSetGMat.m
%	FAP:			genfapData.m	
%	BIQ:			GenBIQ_be_Matlab.m
%
% *Two useful codes for calling routines in LAPACK to compute eigenvalue decomposition
% 	mexeig.c		mex function for calling dsyevd in LAPACK
%				(This code is a modification of mexeig.c in the code SDPNAL)			
% 	MexDSYEVX.c 		mex function for calling dsyevx in LAPACK
%  The above two routines can be used to replace the code "eig" in MATLAB
% -------------------------------------------------------------------------
% 4. License
% 
% See License.txt for the license of this program
%
% -------------------------------------------------------------------------
% 
% 5. Reference:
% @TECHREPORT{AltSDP-WenGoldfarbYin-2009, 
%	author = {Wen, Zaiwen and Goldfarb, Donald and Yin, Wotao}, 
%	title = {Alternating Direction Augmented Lagrangian Methods for semidefinite programming}, 
%	institution = {Dept of IEOR, Columbia University}, 
%	year = {2009} }
%
% -------------------------------------------------------------------------
% 8. The Author
%
% We hope that SDPAD is useful for your application.  If you have
% any bug reports or comments, please feel free to email me at 
%
%   Zaiwen Wen, zw2109@columbia.edu
%
% Enjoy!
% Zaiwen Wen
%
%

